<?xml version="1.0" encoding="iso-8859-1" standalone="yes" ?>
<!-- $Id: form-raweb.xsl,v 1.4 2007/03/08 11:21:48 euzenat Exp $ -->

<xsl:stylesheet version="1.0"
  xml:space="default"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:bib="http://exmo.inrialpes.fr/papers"
  xmlns:datext="http://www.jclark.com/xt/java/java.util.Date"
  xmlns:datexa="xalan://java.util.Date"
  xmlns:java="http://xml.apache.org/xslt/java">

<xsl:import href="common.xsl" />

<xsl:param name="year">2005</xsl:param>             

<xsl:output method="xml" encoding="iso-8859-1" indent="yes"/>

<xsl:strip-space elements="*"/>

<!-- toplevel -->

<xsl:template match="/">
	<xsl:apply-templates select="*"/>
</xsl:template>

<xsl:template match="bibliography">
<biblio id="bibliography">
	<xsl:apply-templates select="reference[date/@year=$year]"/>
</biblio>
</xsl:template>

<!-- DONE -->
<xsl:template match="reference">
    <biblStruct id="{@index}">
      <xsl:choose>
        <xsl:when test="@type='inproceedings'">
	  <xsl:apply-templates select="." mode="analytic" />
	  <xsl:apply-templates select="." mode="edited" />
	</xsl:when>
        <xsl:when test="@type='inbook'">
	  <xsl:apply-templates select="." mode="analytic" />
	  <xsl:apply-templates select="." mode="edited" />
	</xsl:when>
        <xsl:when test="@type='incollection'">
	  <xsl:apply-templates select="." mode="analytic" />
	  <xsl:apply-templates select="." mode="edited" />
	</xsl:when>
        <xsl:when test="@type='article' or @type='specialissue'">
	  <xsl:apply-templates select="." mode="analytic" />
	  <xsl:apply-templates select="." mode="edited" />
	</xsl:when>
        <xsl:when test="@type='techreport' or @type='internalreport'
			or @type='prospective' or
			@type='deliverable'">
	  <xsl:apply-templates select="." mode="monogr" />
	</xsl:when>
        <xsl:when test="@type='proceedings' or
			@type='serialproceedings'">
	  <xsl:apply-templates select="." mode="monogr" />
	</xsl:when>
        <xsl:when test="@type='book' or @type='collection'">
	  <xsl:apply-templates select="." mode="monogr" />
	</xsl:when>
        <xsl:when test="@type='phd'">
	  <xsl:apply-templates select="." mode="monogr" />
	</xsl:when>
        <xsl:when test="@type='mastersthesis'">
	  <xsl:apply-templates select="." mode="monogr" />
	</xsl:when>
        <xsl:when test="@type='manual'">
	  <xsl:apply-templates select="." mode="monogr" />
	</xsl:when>
        <xsl:when test="@type='booklet'">
	  <xsl:apply-templates select="." mode="monogr" />
	</xsl:when>
        <xsl:when test="@type='bulletin' or @type='recension'">
	  <xsl:apply-templates select="." mode="analytic" />
	  <xsl:apply-templates select="." mode="edited" />
	</xsl:when>
        <xsl:when test="@type='lecturenotes'">
	  <xsl:apply-templates select="." mode="monogr" />
	</xsl:when>
        <xsl:when test="@type='motionpictures'">
	  <xsl:apply-templates select="." mode="monogr" />
	</xsl:when>
        <xsl:otherwise>
	  <xsl:apply-templates select="." mode="monogr" />
	</xsl:otherwise>
      </xsl:choose>
      <dateStruct><year><xsl:value-of select="date/@year"/></year></dateStruct>
      <xsl:if test="links"><xsl:apply-templates select="links"/></xsl:if>
      <!-- This was generated by the translation pgm but is useless -->
      <note type="classification"><xsl:call-template name="typedef"/></note>
      <note type="from">year</note>
      <note type="userid"><xsl:value-of select="@index"/></note>
    </biblStruct>
</xsl:template>

<!-- DONE -->
<xsl:template match="reference" mode="analytic">
      <analytic>
        <title level="a"><xsl:value-of select="title/text()"/></title>
	<xsl:choose>
	  <xsl:when test="authors">
	    <xsl:apply-templates select="authors" />
	  </xsl:when>
	  <xsl:when test="editors">
	    <xsl:apply-templates select="editors" />
	  </xsl:when>
	</xsl:choose>
      </analytic>
</xsl:template>

<xsl:template match="reference" mode="edited">
  <monogr>
    <xsl:choose>
    <xsl:when test="in">
      <xsl:apply-templates select="in/*" mode="premonogr"/>
    </xsl:when>
      <xsl:when test="@type='inproceedings'">
	<xsl:apply-templates select="." mode="procname"/>
      </xsl:when>
      <xsl:when test="booktitle">
        <title level="m"><xsl:value-of select="booktitle/text()"/></title>
      </xsl:when>
    </xsl:choose>
    <xsl:if test="type"><note type="typdoc"><xsl:value-of select="type/text()"/></note></xsl:if>
    <imprint>
      <xsl:if test="journal"><xsl:apply-templates select="journal"/></xsl:if>
      <xsl:if test="volume"><biblScope type="volume"><xsl:value-of select="volume/text()"/></biblScope></xsl:if>
      <xsl:if test="number"><biblScope type="number"><xsl:value-of select="number/text()"/></biblScope></xsl:if>
      <xsl:if test="pages"><biblScope type="pages"><xsl:value-of select="pages/text()"/></biblScope></xsl:if>
      <xsl:if test="publisher"><xsl:apply-templates select="publisher"/></xsl:if>
      <xsl:if test="school"><xsl:apply-templates select="school"/></xsl:if>
      <xsl:if test="organization"><xsl:apply-templates select="organization"/></xsl:if>
      <xsl:if test="institution"><xsl:apply-templates select="institution"/></xsl:if>
      <xsl:if test="contract"><xsl:apply-templates select="contract"/></xsl:if>
    </imprint>
  </monogr>
</xsl:template>

<xsl:template match="reference" mode="monogr">
  <monogr>
    <xsl:apply-templates select="." mode="premonogr"/>
    </monogr>
</xsl:template>

<xsl:template match="reference" mode="premonogr">
	<xsl:apply-templates select="authors|editors" />
	<xsl:if test="title">
          <title level="m"><xsl:value-of select="title/text()"/></title>
	</xsl:if>
	<xsl:choose>
	  <xsl:when test="booktitle">
            <title level="m"><xsl:value-of select="booktitle/text()"/></title>
	  </xsl:when>
	  <xsl:when test="@type='inproceedings' or @type='proceedings'">
	    <xsl:apply-templates select="." mode="procname"/>
	  </xsl:when>
	</xsl:choose>
	<xsl:if test="type"><note type="typdoc"><xsl:value-of select="type/text()"/></note></xsl:if>
        <imprint>
	  <dateStruct><year><xsl:value-of select="date/@year"/></year></dateStruct>
	  <xsl:if test="journal"><xsl:apply-templates select="journal"/></xsl:if>
	  <xsl:if test="volume"><biblScope type="volume"><xsl:value-of select="volume/text()"/></biblScope></xsl:if>
	  <xsl:if test="number"><biblScope type="number"><xsl:value-of select="number/text()"/></biblScope></xsl:if>
	  <xsl:if test="publisher"><xsl:apply-templates select="publisher"/></xsl:if>
	  <xsl:if test="school"><xsl:apply-templates select="school"/></xsl:if>
	  <xsl:if test="organization"><xsl:apply-templates select="organization"/></xsl:if>
	  <xsl:if test="institution"><xsl:apply-templates select="institution"/></xsl:if>
	  <xsl:if test="contract"><xsl:apply-templates select="contract"/></xsl:if>
	  <xsl:if test="pages"><biblScope type="pages"><xsl:value-of select="pages/text()"/></biblScope></xsl:if>
	</imprint>
</xsl:template>

<xsl:template name="typedef">
  <xsl:choose>
    <xsl:when test="@type='inproceedings'">inproceedings</xsl:when>
    <xsl:when test="@type='inbook'">inbook</xsl:when>
    <xsl:when test="@type='incollection'">incollection</xsl:when>
    <xsl:when test="@type='article' or @type='specialissue'">article</xsl:when>
    <xsl:when test="@type='techreport' or @type='internalreport' or @type='prospective' or @type='deliverable'">techreport</xsl:when>
    <xsl:when test="@type='proceedings' or @type='serialproceedings'">proceedings</xsl:when>
    <xsl:when test="@type='book' or @type='collection'">book</xsl:when>
    <xsl:when test="@type='phd'">phdthesis</xsl:when>
    <xsl:when test="@type='mastersthesis'">mastersthesis</xsl:when>
    <xsl:when test="@type='manual'">manual</xsl:when>
    <xsl:when test="@type='booklet'">booklet</xsl:when>
    <xsl:when test="@type='bulletin' or @type='recension'">article</xsl:when>
    <xsl:when test="@type='lecturenotes'">misc</xsl:when>
    <xsl:when test="@type='motionpictures'">misc</xsl:when>
    <xsl:otherwise>misc</xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- UPDT -->
<xsl:template match="authors"> <!-- // -->
<author><xsl:apply-templates select="*"/></author>
</xsl:template>

<!-- UPDT2 -->
<xsl:template match="editors">
<editor><xsl:apply-templates select="*" /></editor>
</xsl:template>

<!-- UPDT -->
<xsl:template match="p">
<persName><foreName><xsl:value-of select='@first'/></foreName><surname><xsl:value-of select='@last'/></surname></persName>
</xsl:template>

<xsl:template match="title">
  <xsl:param name="lev" />
<title level="{$lev}"><xsl:value-of select='normalize-space()'/></title>
</xsl:template>

<!-- UPDT -->
<xsl:template match="publisher">
<publisher><orgName><xsl:value-of select='@name'/>
<address><addrLine><xsl:apply-templates select='location'/></addrLine></address></orgName></publisher>
</xsl:template>

<xsl:template match="type">
<btype><xsl:value-of select='normalize-space()'/></btype>
</xsl:template>

<!-- // ISSUE and ORGANISATION ARE NOT ALLWAYS DISPLAYED SPONTANEOUSLY -->
	<!-- xsl:if test='@issue'>
	ISSUE = <xsl:value-of select='@issue'/>,</xsl:if>
        BOOKTITLE = "<xsl:value-of select='@type'/><xsl:choose><xsl:when test="../@language='fr'"><xsl:text> sur </xsl:text></xsl:when><xsl:otherwise><xsl:text> on </xsl:text></xsl:otherwise></xsl:choose><xsl:value-of select='@name'/>, <xsl:apply-templates select='location'/>",<xsl:if test="@sponsor">
	ORGANISATION = "<xsl:value-of select='@sponsor'/>",</xsl:if></xsl:template !-->

<xsl:template match="conference">
<bbooktitle><xsl:choose>
        <xsl:when test="../@language='fr'"><xsl:text>Actes </xsl:text></xsl:when>
			<xsl:otherwise><xsl:text>Proc. </xsl:text></xsl:otherwise>
			</xsl:choose>
                        <xsl:if test="@issue">
                          <xsl:value-of select="@issue"/>
                          <xsl:call-template name="coderank">
                       			<xsl:with-param name="num" select="@issue"/>
                                        <xsl:with-param name="language" select="../@language"/>
                                      </xsl:call-template>
			</xsl:if>
                        <xsl:if test="@sponsor and ../@language='en'">
				<xsl:value-of select="@sponsor"/>
				<xsl:text> </xsl:text>
			</xsl:if>
			<xsl:choose>
				<xsl:when test="@type"><xsl:value-of select="@type"/></xsl:when>
				<xsl:otherwise><xsl:text>conference</xsl:text></xsl:otherwise>
			</xsl:choose>
			<xsl:text> </xsl:text>
                        <xsl:if test="@sponsor and ../@language='fr'">
				<xsl:value-of select="@sponsor"/>
			</xsl:if>
			<xsl:if test="@name">
                          <xsl:choose>
                            <xsl:when test="../@language='fr'"><xsl:text> sur </xsl:text></xsl:when>
			    <xsl:otherwise><xsl:text> on </xsl:text></xsl:otherwise>
                              </xsl:choose>
				<xsl:value-of select="@name"/>
			</xsl:if>
			<xsl:if test="@short">
				<xsl:text> (</xsl:text>
				<xsl:value-of select="@short"/>
				<xsl:text>)</xsl:text>
			</xsl:if>
			<xsl:text>, </xsl:text>
			<xsl:if test="location">
				<xsl:apply-templates select="location"/>
                              </xsl:if></bbooktitle>
</xsl:template>


<xsl:template match="reference" mode="procname">
  <title level="m">
    <xsl:choose>
      <xsl:when test="../@language='fr'"><xsl:text>Actes </xsl:text></xsl:when>
      <xsl:otherwise><xsl:text>Proc. </xsl:text></xsl:otherwise>
    </xsl:choose>
    <xsl:if test="conference/@issue">
      <xsl:value-of select="conference/@issue"/>
      <xsl:call-template name="coderank">
	<xsl:with-param name="num" select="conference/@issue"/>
	<xsl:with-param name="language" select="../@language"/>
      </xsl:call-template>
    </xsl:if>
    <xsl:if test="conference/@sponsor and ../@language='en'">
      <xsl:value-of select="conference/@sponsor"/>
      <xsl:text> </xsl:text>
    </xsl:if>
    <xsl:choose>
      <xsl:when test="conference/@type"><xsl:value-of select="conference/@type"/></xsl:when>
      <xsl:otherwise><xsl:text>conference</xsl:text></xsl:otherwise>
    </xsl:choose>
    <xsl:text> </xsl:text>
    <xsl:if test="conference/@sponsor and ../@language='fr'">
      <xsl:value-of select="conference/@sponsor"/>
    </xsl:if>
    <xsl:if test="conference/@name">
      <xsl:choose>
	<xsl:when test="../@language='fr'"><xsl:text> sur </xsl:text></xsl:when>
	<xsl:otherwise><xsl:text> on </xsl:text></xsl:otherwise>
      </xsl:choose>
      <xsl:value-of select="conference/@name"/>
    </xsl:if>
    <xsl:if test="conference/@short">
      <xsl:text> (</xsl:text>
      <xsl:value-of select="conference/@short"/>
      <xsl:text>)</xsl:text>
    </xsl:if>
    <xsl:text>, </xsl:text>
    <xsl:if test="conference/location">
      <xsl:apply-templates select="conference/location"/>
    </xsl:if>
  </title>
</xsl:template>

<xsl:template match="in">
<bbooktitle><xsl:value-of select='reference/title'/></bbooktitle>
<xsl:if test="reference/journal">
<bjournal><xsl:apply-templates select='reference/journal'/></bjournal>
<bvolume><xsl:apply-templates select='reference/volume'/></bvolume>
<xsl:if test="reference/number">
<bnumber><xsl:apply-templates select='reference/number'/></bnumber>
</xsl:if>
</xsl:if>
<xsl:if test="reference/authors">
<beditor><xsl:apply-templates select='reference/authors/*'/></beditor>
</xsl:if>
<xsl:if test="reference/editors">
<beditor><xsl:apply-templates select='reference/editors/*'/></beditor>
</xsl:if>
	<xsl:apply-templates select='reference/publisher'/>
</xsl:template>

<xsl:template match="trad|trans|red|rev|ed"/>

<!-- UPDT2 -->
<xsl:template match="organization">
<publisher><orgName type="organization"><xsl:value-of select='@name'/>
<address><addrLine><xsl:apply-templates select='location'/></addrLine></address></orgName></publisher>
</xsl:template>

<!-- UPDT2 -->
<xsl:template match="contract">
<publisher><orgName type="organisation"><xsl:value-of select='text()'/></orgName></publisher>
</xsl:template>

<!-- UPDT2 -->
<xsl:template match="institution">
<publisher><orgName type="institution"><xsl:value-of select='@name'/>
<address><addrLine><xsl:apply-templates select='location'/></addrLine></address></orgName></publisher>
</xsl:template>

<!-- UPDT2 -->
<xsl:template match="school">
<publisher><orgName type="school"><xsl:value-of select='@name'/>
<address><addrLine><xsl:apply-templates select='location'/></addrLine></address></orgName></publisher>
</xsl:template>

<xsl:template match="chapter">
<bchapter><xsl:value-of select='normalize-space()'/></bchapter>
</xsl:template>

<xsl:template match="howpublished">
<!--	HOWPUBLISHED = "<xsl:value-of select='normalize-space()'/>",-->
</xsl:template>

<!-- UPDT -->
<xsl:template match="journal">
<title level="j"><xsl:value-of select='@name'/></title>
<xsl:if test="publisher"><xsl:apply-templates select="publisher"/></xsl:if>
</xsl:template>

<xsl:template match="edition">
<!--	EDITION = "<xsl:value-of select='normalize-space()'/>",-->
</xsl:template>

<xsl:template match="volume">
<bvolume><xsl:value-of select='normalize-space()'/></bvolume>
</xsl:template>

<xsl:template match="number">
<bnumber><xsl:value-of select='normalize-space()'/></bnumber>
</xsl:template>

<xsl:template match="pages">
<bpages><xsl:value-of select='normalize-space()'/></bpages>
</xsl:template>

<!--xsl:template match="length">
  <xsl:if test="@unit='p'">
    PAGES = "<xsl:value-of select='normalize-space()'/>",</xsl:if></xsl:template-->

<xsl:template match="date">
	<!--xsl:if test="month">
        MONTH = "<xsl:call-template name="codemonth">
                        	<xsl:with-param name="month" select="@month"/>
                                <xsl:with-param name="language" select="../@language"/>
                              </xsl:call-template>",</xsl:if-->
	<byear><xsl:value-of select='@year'/></byear>
</xsl:template>

<xsl:template match="address">
<baddress><xsl:value-of select='normalize-space()'/></baddress>
</xsl:template>

<xsl:template match="links"><xsl:apply-templates select="url[1]"/></xsl:template>

<xsl:template match="url">
<xsl:if test="not(@status='hidden')">
<ref>
<xsl:attribute name="xlink:href"><xsl:value-of select="@href"/></xsl:attribute>
<xsl:attribute name="location">extern</xsl:attribute>
<xsl:value-of select="@href"/>
</ref>
</xsl:if>
</xsl:template>

<!-- DONE -->
<xsl:template match="series">
<title level="s"><xsl:value-of select='@name'/></title>
</xsl:template>

<xsl:template match="note">
<bnote><xsl:value-of select='normalize-space()'/></bnote>
</xsl:template>

<xsl:template match="comments">
<!--	ANNOTE = "<xsl:value-of select='normalize-space()'/>",-->
</xsl:template>

<xsl:template match="location">
	<xsl:if test="@place"><xsl:value-of select="@place"/><xsl:text>, </xsl:text></xsl:if>
	<xsl:if test="@city"><xsl:value-of select="@city"/></xsl:if>
	<xsl:if test="@country"><xsl:text> (</xsl:text><small>
		<xsl:if test="@state"><xsl:value-of select="@state"/><xsl:text> </xsl:text></xsl:if>
		<xsl:value-of select="@country"/></small>)</xsl:if>
</xsl:template>

<!-- this could be put back now -->
<xsl:template match="areas|softwares|contracts|keywords|abstract|comments|award|errata"/>

<xsl:template match="text()"/>

<xsl:template match="li"><xsl:value-of select="normalize-space()"/></xsl:template>

</xsl:stylesheet>
