/*
 * $Id: Ruleset.java,v 1.7 2004/02/24 15:11:05 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 * Copyright (C) INRIA Rhne-Alpes, 2003.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package fr.fluxmedia.transmorpher.graph;
import fr.fluxmedia.transmorpher.graph.rules.*;

import fr.fluxmedia.transmorpher.utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.utils.TMException;
import fr.fluxmedia.transmorpher.utils.Version;
import fr.fluxmedia.transmorpher.utils.Writer;
import java.io.File;
import java.io.IOException;

import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;

/**
 * Transmorpher graph ruleset definitions
 *
 *@author    Jerome.Euzenat@inrialpes.fr
 *@since     jdk 1.3 / SAX 2.0
 */

public class Ruleset extends TransformationImpl implements Namespaced {

	/**
	 *  a list for the rules
	 */
	protected ArrayList rules = null;
	/**
	 *  List of namespaces
	 */
	protected ArrayList namespaces = null;

	/**
	 *Constructor for the Ruleset object
	 *
	 *@param  name  The name of the Ruleset instance to create
	 *@param  owner  The Transmorpher instance that owns this Ruleset instance
	  */
	public Ruleset(String name, Transmorpher owner) {
		super(name, "Ruleset", owner, 1, 1);
		rules = new ArrayList(10);
		namespaces = new ArrayList(10);
	}

	/**
	 *  Adds a rule to this Ruleset instance
	 *
	 *@param  rule  The rule to add
	 */
	public void addRule(Rule r) {
		rules.add((Object)r);
	}

	/**
	*  Adds a namespace to this Query instance
	 *
	 *@param  name  The namespace to add
	 */
	public void addNamespace(Namespace n) {
		namespaces.add((Object)n);
	}

	/**
	  *  Adds a rule at the specified index to this Ruleset instance
	 *
	 *@param  rule  The rule to Add
	 *@param  index  index in the list
	 */
	public void insertRule(Rule rule, int index) {
	    rules.add(index, (Object)rule);
	}

	/**
	 * Prints the XML expression of the process
	 */
	public void generateXML() {
	    System.out.println("  <ruleset name=\"" + name + "\" type=\"" + type + "\">");
	    for (int i = 0; i < namespaces.size(); i++) {
		((Namespace)namespaces.get(i)).generateXML();
	    }
	    parameters.generateXMLDeclarations();
	    for (int i = 0; i < rules.size(); i++) {
		((Rule)rules.get(i)).generateXML();
	    }
	    System.out.println("  </ruleset>");
	    System.out.println();
	}

	/**
	 * Prints the Java code for the execution of the call
	 *
	 *@param  file     Description of the Parameter
	 *@param  externs  Description of the Parameter
	 */
	public void generateJavaCode(Writer file, LinearIndexedStruct externs) { }

	/**
	 * generate the stylesheet for the Ruleset (in the correct file)
	 *
	 *@param  		reloc         The path where the stylesheet have to be put
	 *@exception  IOException  	When IO errors occur
	 *@exception  IOException  Description of the Exception
	 */
	public void generateXSLTStylesheet(String reloc) throws TMException, IOException {
	    String vPath = getTransmorpher().getFile().getAbsoluteFile().getParent();
	    if (vPath == null) {
		vPath = ".";
	    } else {
		vPath = vPath.replace('\\', '/');
	    }
	    if (reloc != null) {
		vPath = reloc;
	    }
	    //System.out.println("[RuleSEt] vPath = " + vPath +" Reloc = "+ reloc);
	    generateXSLTCode(new Writer(vPath + "/" + name + ".xsl"));
	    attributes.setParameter("file", vPath + "/" + name + ".xsl");
	}

	/**
	 * Prints the xsl for the execution of this Ruleset instance
	 *
	 *@param  file            The writer used for printing in a file
	 *@exception  TMException  Description of the Exception
	 *@exception  IOException  Description of the Exception
	 */
	public void generateXSLTCode(Writer file) throws TMException, IOException {
	    file.writeln("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
	    file.writeln("<xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\" version=\"1.0\"");
	    for (int i = 0; i < namespaces.size(); i++) {
		((Namespace)namespaces.get(i)).generateXSLTCode(file);
	    }
	    file.writeln(" xmlns:regexp=\"http://fr.fluxmedia.transmorpher.stdlib.RegularExpression\">");
	    file.writeln("");
	    file.writeln("<!-- ************************************************************");
	    file.writeln("     Generated by " + Version.NAME + " " + Version.RELEASE);
	    Date date = new Date();
	    file.writeln("     " + date);
	    file.writeln("     See " + Version.URL);
	    file.writeln("     ************************************************************ -->");
	    for (Enumeration e = parameters.getNames(); e.hasMoreElements(); ) {
		String key = (String)e.nextElement();
		file.writeln(2, "<xsl:param name=\"" + key + "\">" + parameters.getParameter(key) + "</xsl:param>");
	    }
	    file.writeln("");
	    file.writeln("  <!-- Copying the root and its attributes -->");
	    file.writeln("  <xsl:template match=\"/\">");
	    file.writeln("    <xsl:apply-templates/>");
	    file.writeln("  </xsl:template>");
	    file.writeln("  <xsl:template match=\"@*\">");
	    file.writeln("    <xsl:attribute name=\"{name()}\"><xsl:value-of select=\".\"/>");
	    file.writeln("    </xsl:attribute>");
	    file.writeln("  </xsl:template>");
	    file.writeln("");
	    file.writeln("  <!-- Copying all elements and attributes -->");
	    file.writeln("  <xsl:template match=\"*|@*|text()\">");
	    file.writeln("    <xsl:copy>");
	    file.writeln("      <xsl:apply-templates select=\"*|@*|text()\"/>");
	    file.writeln("    </xsl:copy>");
	    file.writeln("  </xsl:template>");
	    file.writeln("");
	    file.writeln("<!-- ************************************************************ -->");
	    file.writeln("<!--     End of the general section, here begins the stylesheet   -->");
	    file.writeln("<!-- ************************************************************ -->");
	    file.writeln("");
	    for (int i = 0; i < rules.size(); i++) {
		((Rule)rules.get(i)).generateXSLTCode(file);
	    }//end for
	    file.writeln("</xsl:stylesheet>");
	    file.close();
	}

}

